package edu.stanford.crypto.cs251;

import edu.stanford.crypto.cs251.transactions.*;
import org.bitcoinj.core.*;
import org.bitcoinj.kits.WalletAppKit;
import org.bitcoinj.params.TestNet3Params;
import org.bitcoinj.script.Script;
import org.junit.Assert;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;

/**
 * Created by bbuenz on 23.09.15.
 */
public class ScriptTests {
    //TODO: Change this to use the main net.
    private NetworkParameters networkParameters = new TestNet3Params();
    private WalletAppKit kit = new WalletAppKit(networkParameters, new File("wallet"), "password");
    private static final Logger LOGGER = LoggerFactory.getLogger(ScriptTests.class);

    public void downloadBlockchain() {
        LOGGER.info("Starting to sync blockchain. This might take a few minutes");
        kit.setAutoSave(true);
        kit.startAsync();
        kit.awaitRunning();
        kit.wallet().allowSpendingUnconfirmedTransactions();
        LOGGER.info("Synced blockchain.");
        LOGGER.info("You've got " + kit.wallet().getBalance() + " in your pocket");


    }


    @Test
    public void testPayToPubKey() throws InsufficientMoneyException {
        try (ScriptTransaction payToPubKey = new PayToPubKey(networkParameters, new File("wallet"), "password")) {
            testTransaction(payToPubKey);

        } catch (Exception e) {
            e.printStackTrace();
            Assert.fail(e.getMessage());
        }

    }

    @Test
    public void testPayToPubKeyHash() throws InsufficientMoneyException {
        try (ScriptTransaction payToPubKeyHash = new PayToPubKeyHash(networkParameters, new File("wallet"), "password")) {
            testTransaction(payToPubKeyHash);
        } catch (Exception e) {
            e.printStackTrace();
            Assert.fail(e.getMessage());

        }

    }

    @Test
    public void testLinearEquation() throws InsufficientMoneyException {
        try (LinearEquationTransaction linEq = new LinearEquationTransaction(networkParameters, new File("wallet"), "password")) {
            testTransaction(linEq);
        } catch (Exception e) {
            e.printStackTrace();
            Assert.fail(e.getMessage());

        }

    }

    @Test
    public void testMultiSig() throws InsufficientMoneyException {
        try (ScriptTransaction multiSig = new MultiSigTransaction(networkParameters, new File("wallet"), "password")) {
            testTransaction(multiSig);
        } catch (Exception e) {
            e.printStackTrace();
            Assert.fail(e.getMessage());

        }

    }

    private void testTransaction(ScriptTransaction scriptTransaction) throws InsufficientMoneyException {
        final Script challengeScript = scriptTransaction.createChallengeScript();
        Transaction challengeTransaction = scriptTransaction.createChallengeTransaction(challengeScript, Coin.CENT);
        TransactionOutput challengeOutput = challengeTransaction.getOutputs().stream().filter(to -> to.getScriptPubKey().equals(challengeScript)).findAny().get();
        Transaction redemptionTransaction = scriptTransaction.createUnsignedRedemptionTransaction(challengeOutput, scriptTransaction.getReceiveAddress());
        Script redemptionScript = scriptTransaction.createRedemptionScript(redemptionTransaction);
        scriptTransaction.testScript(challengeScript, redemptionScript, redemptionTransaction);
        redemptionTransaction.getInput(0).setScriptSig(redemptionScript);
        scriptTransaction.sendTransaction(challengeTransaction);
        scriptTransaction.sendTransaction(redemptionTransaction);
    }

    @Test
    public void sendMoneyBackToFaucet() throws AddressFormatException, InsufficientMoneyException {
        downloadBlockchain();
        Transaction transaction = kit.wallet().createSend(new Address(networkParameters, "n2eMqTT929pb1RDNuqEnxdaLau1rxy3efi"), kit.wallet().getBalance().subtract(Coin.MILLICOIN));
        kit.wallet().commitTx(transaction);
        kit.peerGroup().broadcastTransaction(transaction);
        kit.stopAsync();
        kit.awaitTerminated();
    }

    @Test
    public void printAddress() {
        downloadBlockchain();
        LOGGER.info("Your address is {}", kit.wallet().currentReceiveAddress());
        kit.stopAsync();
        kit.awaitTerminated();
    }

}
